#ifndef __REFCOUNT_H__
#define __REFCOUNT_H__

#if 1
#include "sys/atomic.h"

class ReferenceCounted 
{
public:
	ReferenceCounted() { nReferences = 0; }
	volatile atomic_t nReferences;
private:
//	ReferenceCounted( const ReferenceCounted & );
	ReferenceCounted &operator=( const ReferenceCounted & );
};

template <class T> class Reference 
{
public:
	// Reference constructors
	Reference( T *p = 0 ) 
	{
		ptr = p;
		if ( ptr ) atomic_add(&ptr->nReferences,1);
	}
	Reference( const Reference<T> &r ) 
	{
		ptr = r.ptr;
		if ( ptr ) atomic_add(&ptr->nReferences,1);
	}
	// Reference assignment operators
	Reference &operator=( const Reference<T> &r ) 
	{
		return operator=( r.ptr );
	}
	Reference &operator=( T *p ) 
	{
		if (p) atomic_add(&p->nReferences,1);
		if ( ptr && atomic_add(&ptr->nReferences,-1) == 1 ) delete ptr;
		ptr = p;
		return *this;
	}
	// Reference destructor
	~Reference() 
	{
		if ( ptr && atomic_add(&ptr->nReferences,-1) == 1 )
			delete ptr;
	}
	// Reference operators
	const T *operator->() const { return ptr; }
	T *operator->() { return ptr; }
	const T *get() const { return ptr; }
	T *get() { return ptr; }
	operator bool() const { return ptr != NULL; }
	T *edit()
	{
		if ( !ptr )
		{
			ptr = new T();
			ptr->nReferences = 1;
		}
		else if ( ptr->nReferences > 1 )
		{
			ptr->nReferences--;
			ptr = new T( *ptr );
			ptr->nReferences = 1;
		}
		return ptr;
	}
private:
	T *ptr;
};
#else
class ReferenceCounted 
{
public:
	ReferenceCounted() { nReferences = 0; }
	int nReferences;
private:
	ReferenceCounted &operator=( const ReferenceCounted & );
};

template <class T> class Reference 
{
public:
	// Reference constructors
	Reference( T *p = 0 ) 
	{
		ptr = p;
		if ( ptr ) ptr->nReferences++;
	}
	Reference( const Reference<T> &r ) 
	{
		ptr = r.ptr;
		if ( ptr ) ptr->nReferences++;
	}
	// Reference assignment operators
	Reference &operator=( const Reference<T> &r ) 
	{
		return operator=( r.ptr );
	}
	Reference &operator=( T *p ) 
	{
		if ( p ) p->nReferences++;
		if ( ptr && --ptr->nReferences == 1 ) delete ptr;
		ptr = p;
		return *this;
	}
	// Reference destructor
	~Reference() 
	{
		if ( ptr && --ptr->nReferences == 1 )
			delete ptr;
	}
	// Reference operators
	const T *operator->() const { return ptr; }
	T *operator->() { return ptr; }
	const T *get() const { return ptr; }
	T *get() { return ptr; }
	operator bool() const { return ptr != NULL; }
	T *edit()
	{
		if ( !ptr )
		{
			ptr = new T();
			ptr->nReferences = 1;
		}
		else if ( ptr->nReferences > 1 )
		{
			ptr->nReferences--;
			ptr = new T( *ptr );
			ptr->nReferences = 1;
		}
		return ptr;
	}
private:
	T *ptr;
};
#endif

#endif //__REFCOUNT_H__
