#ifndef __SHADERENVIRONMENT_H__
#define __SHADERENVIRONMENT_H__

#include "vector3.h"
#include "color3.h"
#include "transform.h"
#include "refcountedlist.h"
#include "geometryset.h"

namespace xrt
{
	class Scene;
	class RayDifferential;
	class Light;
	class ShaderGroup;

	class PUBLIC ShaderEnvironment
	{
	public:
		float ncomps;
		float alpha;
		Color3 Ci, Cs, Cl;
		Color3 Oi, Os, Ol;
		float u, v, du, dv, s, t;
		float ug, vg;
		Vector3 P, Ps;
		Vector3 N, Ng;
		Vector3 I, L;
		Vector3 E;
		Vector3 dPdu, dPdv, dNdu, dNdv;
		Vector3 dPdx, dPdy;
		float dudx, dudy, dvdx, dvdy;
		float time;
		float distance;
		const GeometrySet* geometrySet;
		Reference< RefCountedList< Reference< Light > > > activeLights;
		const RayDifferential *ray;
		const Scene *scene; //backdoor to renderer context
		float shadowBias; // FIXME; this should go into Hit
		Transform worldToObject;
		Transform worldToShader;
		std::vector<float> scratch;
		unsigned int highestScratch;
		unsigned int whichDifferential;
		struct Lighting
		{
			Lighting( Vector3 L, Color3 Cl, Color3 Ol, Light* light, void* parameters ): L(L), Cl(Cl), Ol(Ol), light(light), parameters(parameters) {}
			Vector3 L;
			Color3 Cl, Ol;
			void *parameters;
			Light* light;
		};
		std::vector< Lighting > illuminanceCache;
		Light* currentLight;
		void* currentLightParameters;
		std::vector< Reference< Light > >::iterator lightIte;
		std::vector< ShaderEnvironment::Lighting >::iterator cacheIterator;
		unsigned int totalSamples;
		Vector3 position;
		Vector3 axis;
		float angle;
		bool isIlluminanceCacheValid;

		Reference<ShaderGroup> surface;
		Reference<ShaderGroup> displacement;
		Reference<ShaderGroup> volume;
		std::vector<void*> surfaceParameters;
		std::vector<void*> displacementParameters;
		std::vector<void*> volumeParameters;
		void computeDifferentials( const RayDifferential& );
		ShaderEnvironment();
		ShaderEnvironment& operator= ( const ShaderEnvironment& sv );
	};
}
#endif //__SHADERENVIRONMENT_H__
