#ifndef __TRANSFORM_H__
#define __TRANSFORM_H__

#include "vector3.h"
#include "matrix4.h"
#include "bbox3.h"

#include "public.h"

namespace xrt
{
	class Ray;
	class RayDifferential;
	class HPoint3;

	class PUBLIC Transform
	{
	public:
//		Transform() { from = to = new Matrix( 1 ); }
		Transform(): from(Matrix4::Identity()), to(Matrix4::Identity()) {}
		Transform(const float* m): from(Matrix4(m)), to(from.inverse()) {}
		Transform(const Matrix4 &m): from(m), to(from.inverse()) {}
		Transform(const Matrix4 &m, const Matrix4 &inv): from(m), to(inv) {}
		Transform& operator*=(const Transform &);
		EXPORT friend Transform operator*(const Transform &, const Transform&);
		//bool operator==(const Transform &t) const 
		//{
		//	return t.m == m && t.mInv == mInv;
		//}
		//bool operator!=(const Transform &t) const 
		//{
		//	return t.m != m || t.mInv != mInv;
		//}
		bool operator<(const Transform &t) const 
		{
			for (unsigned int i = 0; i < 4; ++i)
				for (unsigned int j = 0; j < 4; ++j) 
				{
					if (from[i][j] < t.from[i][j]) return true;
					if (from[i][j] > t.from[i][j]) return false;
				}
			return false;
		}
		Vector3 transformp (const Vector3&) const;
		void transformp (const Vector3*, Vector3*, int) const;
		void transformp (Vector3* P, int n) const { transformp (P, P, n); }
		Vector3 transformv (const Vector3&) const;
		void transformv (const Vector3*, Vector3*, int) const;
		void transformv (Vector3* P, int n) const { transformv (P, P, n); }
		Vector3 transformn (const Vector3&) const;
		void transformn (const Vector3*, Vector3*, int) const;
		void transformn (Vector3* P, int n) const { transformn (P, P, n); }
		Vector4 transformp (const Vector4&) const;
		void transformp (const Vector4*, Vector4*, int) const;
		void transformp (Vector4* P, int n) const { transformp (P, P, n); }
		Ray operator* ( const Ray & ) const;
		RayDifferential operator* ( const RayDifferential & ) const;
		Matrix4 operator* ( const Matrix4 & ) const;
		BBox3 operator* ( const BBox3 & ) const;
		Transform inverse() const { return Transform(to, from); }
		bool isLeftHanded() const { return from.isLeftHanded(); }
		bool hasIsotropicScale(float& scaleFactor) const { return from.hasIsotropicScale(scaleFactor); }
		const Matrix4& getFrom() const { return from; }
		const Matrix4& getTo() const { return to; }
		static const Transform Identity;
	private:
		friend class MotionTransform;
		Matrix4 from, to;
	};

	EXPORT Transform Translate( float, float, float );
	EXPORT Transform Rotate( float, const Vector3& );
	EXPORT Transform Skew( const float, const Vector3&, const Vector3 & );
	EXPORT Transform Scale( float, float, float );
}


#endif //__TRANSFORM_H__
