void sampleDisk( float u1, u2; output float dx, dy ) 
{
	float r, theta;
	// Map uniform random numbers to $[-1,1]^2$
	float sx = 2 * u1 - 1;
	float sy = 2 * u2 - 1;
	// Map square to $(r,\theta)$
	// Handle degeneracy at the origin
	if (sx == 0.0 && sy == 0.0) 
	{
		dx = 0.0;
		dy = 0.0;
	}
	else
	{
		if (sx >= -sy) 
		{
			if (sx > sy) 
			{
				// Handle first region
				r = sx;
				if (sy > 0.0)
					theta = sy/r;
				else
					theta = 8.0 + sy/r;
			}
			else 
			{
				// Handle second region
				r = sy;
				theta = 2.0 - sx/r;
			}
		}
		else 
		{
			if (sx <= sy) 
			{
				// Handle third region
				r = -sx;
				theta = 4.0 - sy/r;
			}
			else 
			{
				// Handle fourth region
				r = -sy;
				theta = 6.0 + sx/r;
			}
		}
		theta *= PI / 4.0;
		dx = r * cos( theta );
		dy = r * sin( theta );
	}
}

surface
ystsurface (float Ka = 0.0;
     float Kd = 0.0;
     float Ks = 0.0;
     float roughness = 0.0;
     float reflected = 0.;
     float reflection_radius = 0.;
     float reflection_samples = 0.;
     float transmitted = 0.;
     float index = 1.0;)
{
  normal Nn = faceforward (normalize(N),I);
  normal In = normalize( I );
  color r,tr;
     
  float depth;
  rayinfo("depth", depth);
	
  if (reflected > .001)
  {
	vector reflection = reflect(I, Nn);
	if ( reflection_radius > 0.001 && depth < 2 ) // avoid combinatorial explosion
	{
		reflection = normalize( reflection );
		
		/* Construct orthogonal components to reflection */
		vector uoffset = normalize ( vector ( 
			zcomp( reflection ) - ycomp( reflection ),
			xcomp( reflection ) - zcomp( reflection ),
			ycomp( reflection ) - xcomp( reflection ) ) );
			
		vector voffset = reflection ^ uoffset;
		
		vector direction;
		float i;
		for ( i = 0; i < reflection_samples; i += 1.0 )
		{
		/* Add a random offset to the smooth reflection vector */
			float dx, dy;
			sampleDisk( random(), random(), dx, dy );
			direction = reflection + reflection_radius * ( dx * uoffset + dy * voffset );
			r += reflected * color trace( P, direction );
		}
		r /= reflection_samples;
	}
	else
		r = reflected * color trace (P, reflection);
  }
  if (transmitted > .001) 
  {
   vector T = refract(In, Nn, (N.I)>0 ? index : 1/index ); 
   if ( length( T ) != 0.0 )
   tr = transmitted * color trace (P, T);
  }
     
  color C = Ka * Cs;
  
  illuminance( P, Nn, PI/2 )
  {
	vector Ln = normalize(L);
	C += Cl * ( Cs * Kd * Ln.Nn + Ks * pow( Ln.reflect( In, Nn ), roughness ) );
  }
  
  Oi = Os;
  Ci = Os * ( C + r + tr );
}